#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Copyright (c) xFusion Digital Technologies Co., Ltd. 2016-2019. All rights reserved.
Description: register alarm
Author: yangbin
"""

from ctypes import CDLL
from ctypes import CFUNCTYPE
from ctypes import c_int
from ctypes import c_void_p
from ctypes import byref
from ctypes import c_char_p
from ctypes import c_longlong
from ctypes import Structure

LIB = CDLL("/usr/lib64/libregalarm.so")

CMPFUNC = CFUNCTYPE(c_int, c_void_p)


class Hook(Structure):
    """
    Description : callback hook
    """
    _fields_ = [("pfunc", CMPFUNC)]

    def __init__(self, func):
        self.pfunc = func
CFUN = None


def getid(palarm):
    """
    Description : get alarm id
    Notes       : function doesn't support multithreading
                  and isn't signal security
    """
    c_palarm = c_void_p(palarm)
    return LIB.OS_alarm_getid(c_palarm)


def getlevel(palarm):
    """
    Description : get alarm level
    Notes       : function doesn't support multithreading
                  and isn't signal security
    """
    c_palarm = c_void_p(palarm)
    return  LIB.OS_alarm_getlevel(c_palarm)


def gettype(palarm):
    """
    Description : get alarm type
    Notes       : function doesn't support multithreading
                  and isn't signal security
    """
    c_palarm = c_void_p(palarm)
    return LIB.OS_alarm_gettype(c_palarm)


def gettime(palarm):
    """
    Description : get alarm time
    Notes       : function doesn't support multithreading
                  and isn't signal security
    """
    c_palarm = c_void_p(palarm)
    LIB.OS_alarm_gettime.restype = c_longlong
    return LIB.OS_alarm_gettime(c_palarm)


def getdesc(palarm):
    """
    Description : get alarm description
    Notes       : function doesn't support multithreading
                  and isn't signal security
    """
    c_palarm = c_void_p(palarm)
    LIB.OS_alarm_getdesc.restype = c_char_p
    return LIB.OS_alarm_getdesc(c_palarm)


def getexdesc(palarm):
    """
    Description : get alarm extension description
    Notes       : function doesn't support multithreading
                  and isn't signal security
    """
    c_palarm = c_void_p(palarm)
    LIB.OS_alarm_getexdesc.restype = c_char_p
    return LIB.OS_alarm_getexdesc(c_palarm)


def getsource(palarm):
    """
    Description : get alarm sourec according to the alarmid
    Notes       : function doesn't support multithreading
                  and isn't signal security
    """
    alarm_id = getid(palarm)
    if alarm_id >= 1200:
        return "Hypervisor"
    else:
        return "OS"


def osalarmreg(myhook):
    """
    Description : register alarm
    Notes       : function doesn't support multithreading
                  and isn't signal security.
                  use global CFUN and Hook to save address
                  for callback hook
    """
    global CFUN
    CFUN = CMPFUNC(myhook)
    _hook = Hook(CFUN)
    return  LIB.OS_alarm_Register(byref(_hook))


def osalarmunreg():
    """
    Description : unregister alarm
    Notes       : function doesn't support multithreading
                  and isn't signal security
    """
    LIB.OS_alarm_UnRegister()


def osalarmresubmit(flag):
    """
    Description : resubmit alarm
    Notes       : function doesn't support multithreading
                  and isn't signal security
    """
    c_flag = c_int(flag)
    return LIB.OS_alarm_resubmit(c_flag)


def osstopresubmit():
    """
    Description : stop resubmit alarm
    Notes       : function doesn't support multithreading
                  and isn't signal security
    """
    return LIB.OS_alarm_resubmit_stop()
